#define STRICT
#include <windows.h>
#include <windowsx.h>
#pragma hdrstop

#include <stdio.h>
#include <commctrl.h>

#include "WebMask.h"
#include "Dialog.h"
#include "Resource.h"

bool subclassed=false;

static DialogPage::HelpTopics helpIds[]={
   {ID_LB_FILTERS,      "auto.filterslist"},
   {ID_B_NEW,           "auto.new"},
   {ID_B_DELETE,        "auto.delete"},
   {ID_B_COPY,          "auto.copy"},
   {ID_B_SAVE,          "auto.save"},
   {ID_B_LOAD,          "auto.merge"},
   {ID_T_HOST,          "auto.host"},
   {ID_EB_FILTER_NAME,  "auto.host"},
   {ID_T_PATH,          "auto.path"},
   {ID_RB_EXACT,        "auto.exact"},
   {ID_T_HTTP_EXACT,    "auto.exact"},
   {ID_EB_PATH_EXACT,   "auto.exact"},
   {ID_RB_PARTIAL,      "auto.partial"},
   {ID_T_HTTP_PARTIAL,  "auto.partial"},
   {ID_EB_PATH_START,   "auto.partial"},
   {ID_T_HTTP_DOTS,     "auto.partial"},
   {ID_EB_PATH_END,     "auto.partial"}
};

class FiltersDialogPage:public DialogPage{
   char oldFile[256];
   HWND myHwnd;
   Filter*lbFilter;
   FilterList*propFilters;
   void readFilter(HWND);
   void addItem(HWND,Filter*);
   void delItem(HWND,Filter*);
   void renItem(HWND,Filter*);
   void setFilterStr(HWND,char*&strptr,int id);
   Filter*currentFilter(HWND);

   static WNDPROC oldNameWndProc;
   static LRESULT CALLBACK nameWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
   static HWND nextbox;
   static void addToDlg(HWND hwnd,DWORD user,Filter*f);
public:
   FiltersDialogPage():DialogPage(DIALOG_FILTERS,ICON_FILTERS,::helpIds,
      sizeof(::helpIds)/sizeof(*::helpIds),"filters","filters.html"),
         lbFilter(NULL),propFilters(NULL){*oldFile='\0';}
   virtual void save();
   virtual void load();
   virtual void enable(HWND);
   virtual BOOL init(HWND,HWND,LPARAM);
   virtual void exit();
   virtual void command(HWND hwnd,int id,HWND hwndCtl,UINT notify);
   virtual int apply(HWND hwnd,int,LPNMHDR pnmhdr);
};

HWND FiltersDialogPage::nextbox;

void saveWff(HWND,FilterList*propFilters,char*filename)throw(char*){
   HANDLE file=CreateFile(filename,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,
                  FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,
                  NULL);
   if(file!=INVALID_HANDLE_VALUE){
      DWORD n;
      WriteFile(file,"WebMask Filter File 0\r\n",23,&n,NULL);
      for(Filter*f=propFilters->f;f;f=f->next){
         WriteFile(file,"\r\nexact=",8,&n,NULL);
         WriteFile(file,"01"+(f->pathType==Filter::EXACT),1,&n,NULL);
         WriteFile(file,"\r\nhost=",7,&n,NULL);
         WriteFile(file,f->name,strlen(f->name),&n,NULL);
         WriteFile(file,"\r\npathExact=",12,&n,NULL);
         WriteFile(file,f->exact,strlen(f->exact),&n,NULL);
         WriteFile(file,"\r\npathStart=",12,&n,NULL);
         WriteFile(file,f->start,strlen(f->start),&n,NULL);
         WriteFile(file,"\r\npathEnd=",10,&n,NULL);
         WriteFile(file,f->end,strlen(f->end),&n,NULL);
         WriteFile(file,"\r\n",2,&n,NULL);
      }
      CloseHandle(file);
      return;
   }
   throw "Failed to create file";
}

const char*nextline(const char*&ptr,unsigned int&n,unsigned int&i){
   for(i=0;i<n;++i){
      if(ptr[i]=='\r'||ptr[i]=='\n') break;
   }
   const int x=1+(i+1<n&&ptr[i]=='\r'&&ptr[i+1]=='\n');
   const char*p=ptr;
   n-=i+x;ptr+=i+x;
   while(i&&*p==' '){++p;--i;}
   const char*p2=ptr-x;
   while(i&&p2[-1]==' '){--p2;--i;}
   return(p);
}

void FiltersDialogPage::addToDlg(HWND hwnd,DWORD user,Filter*f){
   FiltersDialogPage*fdp=(FiltersDialogPage*)user;
   fdp->addItem(GetDlgItem(hwnd,ID_LB_FILTERS),f);
}

int loadWff(HWND hwnd,FilterList*propFilters,char*filename,void(*fn)(HWND,DWORD,Filter*)=NULL,DWORD user=0){
   int nChanged=0;
   HANDLE file=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,NULL,
                  OPEN_EXISTING,
                  FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,
                  NULL);
   if(file!=INVALID_HANDLE_VALUE){
      BY_HANDLE_FILE_INFORMATION bhfi;
      GetFileInformationByHandle(file,&bhfi);
      HANDLE mapping=CreateFileMapping(file,NULL,PAGE_READONLY,
                        0,0,NULL);
      if(mapping){
         const char*const file=(const char*)MapViewOfFile(mapping,FILE_MAP_READ,0,0,0);
         if(file){
            const char*ptr=file;
            unsigned int n=bhfi.nFileSizeLow;
            if(n>=23&&!strncmp(ptr,"WebMask Filter File 0\r\n",23)){
               ptr+=23;n-=23;
               while(n){
                  unsigned int i;
                  const char*p;
                  do{
                     if(!n) goto loaded;
                     p=nextline(ptr,n,i);
                  }while(!i);

                  if(i!=7||strncmp(p,"exact=",6)) break;
                  Filter::PathType pathType=(p[6]=='1'?Filter::EXACT:Filter::PARTIAL);

                  p=nextline(ptr,n,i);
                  if(i<5||strncmp(p,"host=",5)) break;
                  const char*name=p+5;int namelen=i-5;

                  p=nextline(ptr,n,i);
                  if(i<10||strncmp(p,"pathExact=",10)) break;
                  const char*exact=p+10;int exactlen=i-10;

                  p=nextline(ptr,n,i);
                  if(i<10||strncmp(p,"pathStart=",10)) break;
                  const char*start=p+10;int startlen=i-10;

                  p=nextline(ptr,n,i);
                  if(i<8||strncmp(p,"pathEnd=",8)) break;
                  const char*end=p+8;int endlen=i-8;

                  bool same=false;
                  for(Filter*f=propFilters->f;f;f=f->next){
                     if(pathType==f->pathType&&!(
                          strnicmp(name ,f->name ,namelen )||(name ?f->name [namelen ]:(f->name &&*f->name ))
                        ||strncmp (exact,f->exact,exactlen)||(exact?f->exact[exactlen]:(f->exact&&*f->exact))
                        ||strncmp (start,f->start,startlen)||(start?f->start[startlen]:(f->start&&*f->start))
                        ||strncmp (end  ,f->end  ,endlen  )||(end  ?f->end  [endlen  ]:(f->end  &&*f->end  ))))
                     {
                        same=true;
                        break;
                     }
                  }
                  if(!same){
                     propFilters->add(name,namelen,pathType,
                                      exact,exactlen,start,startlen,end,endlen);

                     if(fn){
                        fn(hwnd,user,propFilters->f);
                        PropSheet_Changed(hPropWnd,hwnd);
                     }
                     ++nChanged;
                  }
               }
            loaded:
            }else throw "Not a WebMask Filter file";
            UnmapViewOfFile(file);
         }else throw "Failed to map file to memory";
         CloseHandle(mapping);
      }else throw "Failed to create file mapping";
      CloseHandle(file);
   }else throw "Failed to open file";
   return(nChanged);
}

void FiltersDialogPage::load(){
   if(activeFilters) delete(activeFilters);
   activeFilters=new FilterList;

   HKEY wmKey;
   if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                   KEY_READ,&wmKey)!=ERROR_SUCCESS)
      wmKey=NULL;
   LoadRegStringDef(wmKey,"filterFile",filterFile,256,"_webmask.wff");
   if(wmKey) RegCloseKey(wmKey);

   if(*filterFile&&!*oldFile) strcpy(oldFile,filterFile);

   if(*filterFile){
      char olddir[1024];
      GetCurrentDirectory(1024,olddir);
      char newdir[1024];
      GetModuleFileName(NULL,newdir,1024);
      char*end=newdir;
      for(char*ptr=newdir;*ptr;++ptr)
         if(*ptr=='\\'||*ptr==':'||*ptr=='/') end=ptr;
      if(end!=newdir&&(*end=='\\'||*end=='/')&&end[-1]==':') ++end;
      *end='\0';
      SetCurrentDirectory(newdir);
      try{
         loadWff(NULL,activeFilters,filterFile);
      }catch(...){}
      SetCurrentDirectory(olddir);
   }else{
      HKEY filterKey;
      if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask\\filters",0,
                      KEY_READ,&filterKey)==ERROR_SUCCESS){
         char keyName[512];
         DWORD keylen;
         for(DWORD n=0;(keylen=512),RegEnumKeyEx(filterKey,n,keyName,&keylen,NULL,NULL,NULL,NULL)==ERROR_SUCCESS;++n){
            HKEY itemKey;
            if(RegOpenKeyEx(filterKey,keyName,0,KEY_READ,&itemKey)==ERROR_SUCCESS){
               char host[256];
               DWORD exact;
               char path[512]="";
               char start[512]="";
               char end[512]="";
               if( LoadRegString(itemKey,"host"     ,host ,256)
                && LoadRegDword (itemKey,"exact"    ,exact    )
                &&(LoadRegString(itemKey,"pathExact",path ,512)||!exact)
                &&(LoadRegString(itemKey,"pathStart",start,512)||exact)
                &&(LoadRegString(itemKey,"pathEnd"  ,end  ,512)||exact))
                  activeFilters->add(host,
                                     exact?Filter::EXACT:Filter::PARTIAL,
                                     path,start,end);
               RegCloseKey(itemKey);
            }
         }
         RegCloseKey(filterKey);
      }
   }
}

void FiltersDialogPage::save(){
   HKEY filterKey;
   if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask\\filters",0,
                     NULL,REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_WRITE,NULL,&filterKey,NULL)==ERROR_SUCCESS){
      char keyName[512];
      DWORD keylen;
      for(;(keylen=512),RegEnumKeyEx(filterKey,0,keyName,&keylen,NULL,NULL,NULL,NULL)==ERROR_SUCCESS;)
         RegDeleteKey(filterKey,keyName);
      if(!*filterFile){
         for(Filter*f=activeFilters->f;f;f=f->next){
            int n=0;
            HKEY itemKey=NULL;
            DWORD disp;
            do{
               if(n) wsprintf(keyName,"%s %u",f->name,n);
               else strcpy(keyName,f->name);
               if(RegCreateKeyEx(filterKey,keyName,0,NULL,REG_OPTION_NON_VOLATILE,
                                 KEY_READ|KEY_WRITE,NULL,&itemKey,&disp)!=ERROR_SUCCESS){
                  itemKey=NULL;
                  break;
               }
               ++n;
               if(disp!=REG_CREATED_NEW_KEY) RegCloseKey(itemKey);
            }while(disp!=REG_CREATED_NEW_KEY);
            SaveRegString(itemKey,"host"     ,f->name);
            SaveRegDword (itemKey,"exact"    ,(f->pathType==Filter::EXACT));
            SaveRegString(itemKey,"pathExact",f->exact);
            SaveRegString(itemKey,"pathStart",f->start);
            SaveRegString(itemKey,"pathEnd"  ,f->end  );
            if(itemKey) RegCloseKey(itemKey);
         }
      }
      RegCloseKey(filterKey);
   }
   HKEY wmKey;
   if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                     NULL,REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_WRITE,NULL,&wmKey,NULL)==ERROR_SUCCESS){
      if(*filterFile){
         char olddir[1024];
         GetCurrentDirectory(1024,olddir);
         char newdir[1024];
         GetModuleFileName(NULL,newdir,1024);
         char*end=newdir;
         for(char*ptr=newdir;*ptr;++ptr)
            if(*ptr=='\\'||*ptr==':'||*ptr=='/') end=ptr;
         if(end!=newdir&&(*end=='\\'||*end=='/')&&end[-1]==':') ++end;
         *end='\0';
         SetCurrentDirectory(newdir);
         try{
            saveWff(NULL,activeFilters,filterFile);
         }catch(...){}
         SetCurrentDirectory(olddir);
         RegDeleteKey(wmKey,"filters");
      }
      SaveRegString(wmKey,"filterFile",filterFile);
      RegCloseKey(wmKey);
   }
}

void FiltersDialogPage::readFilter(HWND hwnd){
   Edit_SetText(GetDlgItem(hwnd,ID_EB_FILTER_NAME),lbFilter?lbFilter->name:"");
   Edit_SetText(GetDlgItem(hwnd,ID_EB_PATH_EXACT ),lbFilter?lbFilter->exact:"");
   Edit_SetText(GetDlgItem(hwnd,ID_EB_PATH_START ),lbFilter?lbFilter->start:"");
   Edit_SetText(GetDlgItem(hwnd,ID_EB_PATH_END   ),lbFilter?lbFilter->end:"");

   bool exact=lbFilter&&(lbFilter->pathType==Filter::EXACT);
   Button_SetCheck(GetDlgItem(hwnd,ID_RB_EXACT  ),lbFilter&& exact);
   Button_SetCheck(GetDlgItem(hwnd,ID_RB_PARTIAL),lbFilter&&!exact);
}

void FiltersDialogPage::enable(HWND hwnd){
   BOOL details=(BOOL)lbFilter;

   Button_Enable(GetDlgItem(hwnd,ID_B_DELETE),details);
   Button_Enable(GetDlgItem(hwnd,ID_B_COPY),details);

   Static_Enable(GetDlgItem(hwnd,ID_GB_DETAILS),details);

   Static_Enable(GetDlgItem(hwnd,ID_T_HOST        ),details);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_FILTER_NAME),details);

   Static_Enable(GetDlgItem(hwnd,ID_T_PATH    ),details);
   Static_Enable(GetDlgItem(hwnd,ID_RB_EXACT  ),details);
   Static_Enable(GetDlgItem(hwnd,ID_RB_PARTIAL),details);

   BOOL exact=details&&Button_GetCheck(GetDlgItem(hwnd,ID_RB_EXACT));
   Static_Enable(GetDlgItem(hwnd,ID_T_HTTP_EXACT ),exact);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_PATH_EXACT),exact);

   BOOL partial=details&&Button_GetCheck(GetDlgItem(hwnd,ID_RB_PARTIAL));
   Static_Enable(GetDlgItem(hwnd,ID_T_HTTP_PARTIAL),partial);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_PATH_START ),partial);
   Static_Enable(GetDlgItem(hwnd,ID_T_HTTP_DOTS   ),partial);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_PATH_END   ),partial);

   if(exact) nextbox=GetDlgItem(hwnd,ID_EB_PATH_EXACT);
   if(partial) nextbox=GetDlgItem(hwnd,ID_EB_PATH_START);
}

void FiltersDialogPage::addItem(HWND hwnd,Filter*f){
   char buf[1024];
   if(f->pathType==Filter::EXACT)
      wsprintf(buf,"http://%s/%s",f->name?f->name:"",f->exact?f->exact:"");
   else
      wsprintf(buf,"http://%s/%s ... %s",f->name?f->name:"",f->start?f->start:"",f->end?f->end:"");
   propFilters->setNum(f,ListBox_AddString(hwnd,buf));
}

void FiltersDialogPage::delItem(HWND hwnd,Filter*f){
   ListBox_DeleteString(hwnd,f->num);
   propFilters->delNum(f->num);
}

void FiltersDialogPage::renItem(HWND hwnd,Filter*f){
   int n=ListBox_GetCurSel(hwnd);
   bool set=(n!=LB_ERR&&n==f->num);
   delItem(hwnd,f);
   addItem(hwnd,f);
   if(set) ListBox_SetCurSel(hwnd,f->num);
}

Filter*FiltersDialogPage::currentFilter(HWND hwnd){
   int n=ListBox_GetCurSel(hwnd);
   if(n==LB_ERR) return(NULL);
   Filter*f=propFilters->f;
   while(f){
      if(f->num==n) return(f);
      f=f->next;
   }
   return(NULL);
}

void FiltersDialogPage::exit(){
   subclassed=false;
   SetWindowLong(GetDlgItem(myHwnd,ID_EB_FILTER_NAME),GWL_WNDPROC,LONG(nameWndProc));

   lbFilter=NULL;
   delete(propFilters);
   propFilters=NULL;
}

BOOL FiltersDialogPage::init(HWND hwnd,HWND,LPARAM){
   myHwnd=hwnd;

   lbFilter=NULL;
   propFilters=new FilterList(*activeFilters);

   HWND lbWnd=GetDlgItem(hwnd,ID_LB_FILTERS);

   Filter*f=propFilters->f;
   while(f){
      addItem(lbWnd,f);
      f=f->next;
   }

   lbFilter=currentFilter(lbWnd);

   readFilter(hwnd);

   HWND hwndName=GetDlgItem(hwnd,ID_EB_FILTER_NAME);
   oldNameWndProc=WNDPROC(GetWindowLong(hwndName,GWL_WNDPROC));
   SetWindowLong(hwndName,GWL_WNDPROC,LONG(nameWndProc));

   if(*filterFile) strcpy(oldFile,filterFile);
   else strcpy(oldFile,"_webmask.wff");

   Button_SetCheck(GetDlgItem(hwnd,ID_CB_USEFILE),bool(*filterFile));
   if(*filterFile&&!*oldFile) strcpy(oldFile,filterFile);

   enable(hwnd);
   subclassed=true;
   return(TRUE);
}

WNDPROC FiltersDialogPage::oldNameWndProc;

LRESULT CALLBACK FiltersDialogPage::nameWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam){
   switch(msg){
      case WM_CHAR:{
         char c=char(wParam);
         if(c=='/'){
            char buf[256];
            CallWindowProc(oldNameWndProc,hwnd,WM_GETTEXT,WPARAM(256),LPARAM(buf));
            DWORD start;
            DWORD end;
            CallWindowProc(oldNameWndProc,hwnd,EM_GETSEL,WPARAM(&start),LPARAM(&end));
            if(!buf[end]&&(!start||!strncmp(buf,"http:",start))){
               CallWindowProc(oldNameWndProc,hwnd,WM_SETTEXT,WPARAM(0),LPARAM(buf+end));
               CallWindowProc(oldNameWndProc,hwnd,EM_SETSEL,WPARAM(0),LPARAM(end-start));
               return(FALSE);
            }else{
               if(start!=end){
                  int x=start;
                  while(buf[start++]=buf[end++]) ;
                  CallWindowProc(oldNameWndProc,hwnd,WM_SETTEXT,WPARAM(0),LPARAM(buf));
                  CallWindowProc(oldNameWndProc,hwnd,EM_SETSEL,WPARAM(x),LPARAM(x));
               }
               SetFocus(nextbox);
               CallWindowProc(oldNameWndProc,nextbox,EM_SETSEL,WPARAM(0),LPARAM(-1));
               return(FALSE);
            }
         }
      }
   }
   return(CallWindowProc(oldNameWndProc,hwnd,msg,wParam,lParam));
}

void FiltersDialogPage::setFilterStr(HWND hwnd,char*&strptr,int id){
   HWND hwndEdit=GetDlgItem(hwnd,id);
   if(strptr) delete[](strptr);
   int l;
   strptr=new char[l=1+Edit_GetTextLength(hwndEdit)];
   Edit_GetText(hwndEdit,strptr,l);
   renItem(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter);
}

void FiltersDialogPage::command(HWND hwnd,int id,HWND hwndCtl,UINT notify){
   static bool recurse=false;
   switch(id){
      case ID_EB_FILTER_NAME:
         if(subclassed&&notify==EN_UPDATE&&!recurse){
            char buf[256];
            CallWindowProc(oldNameWndProc,hwndCtl,WM_GETTEXT,WPARAM(256),LPARAM(buf));
            DWORD start;
            DWORD end;
            CallWindowProc(oldNameWndProc,hwndCtl,EM_GETSEL,WPARAM(&start),LPARAM(&end));
            char*ptr=buf;
            for(;;){
               if(*ptr=='/') ++ptr;
               else if(!strncmp(ptr,"http:/",6)) ptr+=6;
               else break;
            }
            if(ptr!=buf){
               start-=ptr-buf;
               end-=ptr-buf;
               CallWindowProc(oldNameWndProc,hwndCtl,WM_SETTEXT,WPARAM(0),LPARAM(ptr));
               CallWindowProc(oldNameWndProc,hwndCtl,EM_SETSEL,WPARAM(start),LPARAM(end));
            }
            char*p0=ptr;
            while(*ptr){
               if(*ptr++=='/'){
                  ptr[-1]='\0';
                  if(start>ptr-p0) start=ptr-p0;
                  if(end>ptr-p0) end=ptr-p0;
                  CallWindowProc(oldNameWndProc,hwndCtl,WM_SETTEXT,WPARAM(0),LPARAM(p0));
                  CallWindowProc(oldNameWndProc,hwndCtl,EM_SETSEL,WPARAM(start),LPARAM(end));
                  SetFocus(nextbox);
                  CallWindowProc(oldNameWndProc,nextbox,WM_SETTEXT,WPARAM(0),LPARAM(ptr));
                  int l=strlen(ptr);
                  CallWindowProc(oldNameWndProc,nextbox,EM_SETSEL,WPARAM(l),LPARAM(l));
                  break;
               }
            }
         }else if(notify==EN_CHANGE&&!recurse){
            if(lbFilter) setFilterStr(hwnd,lbFilter->name,id);
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      case ID_EB_PATH_EXACT:
         if(notify==EN_CHANGE&&!recurse){
            if(lbFilter) setFilterStr(hwnd,lbFilter->exact,id);
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      case ID_EB_PATH_START:
         if(notify==EN_CHANGE&&!recurse){
            if(lbFilter) setFilterStr(hwnd,lbFilter->start,id);
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      case ID_EB_PATH_END:
         if(notify==EN_CHANGE&&!recurse){
            if(lbFilter) setFilterStr(hwnd,lbFilter->end,id);
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      case ID_RB_EXACT:
      case ID_RB_PARTIAL:
         if(lbFilter&&!recurse){
            lbFilter->pathType=(id==ID_RB_EXACT?Filter::EXACT:Filter::PARTIAL);
            renItem(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter);
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      case ID_LB_FILTERS:
         switch(notify){
            case LBN_SELCHANGE:
            case LBN_SELCANCEL:
               recurse=true;
               lbFilter=currentFilter(hwndCtl);
               readFilter(hwnd);
               enable(hwnd);
               recurse=false;
         }
         break;
      case ID_B_NEW:
         propFilters->add("",Filter::PARTIAL,"","","");
         lbFilter=propFilters->f;
         addItem(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter);
         ListBox_SetCurSel(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter->num);
         readFilter(hwnd);
         PropSheet_Changed(hPropWnd,hwnd);
         break;
      case ID_B_DELETE:
         if(!lbFilter) break;
         delItem(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter);
         if(lbFilter==propFilters->f) propFilters->f=lbFilter->next;
         delete(lbFilter);
         lbFilter=currentFilter(hwndCtl);
         if(lbFilter) ListBox_SetCurSel(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter->num);
         readFilter(hwnd);
         PropSheet_Changed(hPropWnd,hwnd);
         break;
      case ID_B_COPY:
         if(!lbFilter) break;
         propFilters->add(lbFilter->name,lbFilter->pathType,
                          lbFilter->exact,lbFilter->start,lbFilter->end);
         lbFilter=propFilters->f;
         addItem(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter);
         ListBox_SetCurSel(GetDlgItem(hwnd,ID_LB_FILTERS),lbFilter->num);
         readFilter(hwnd);
         PropSheet_Changed(hPropWnd,hwnd);
         break;

      case ID_B_SAVE:{
         char filename[512];
         *filename='\0';
         OPENFILENAME of;
         of.lStructSize=sizeof(OPENFILENAME);
         of.hwndOwner=hwnd;
         of.hInstance=NULL;
         of.lpstrFilter="WebMask filters (*.wff)\0*.wff\0All Files (*.*)\0*.*\0";
         of.lpstrCustomFilter=NULL;
         of.nMaxCustFilter=0;
         of.nFilterIndex=1;
         of.lpstrFile=filename;
         of.nMaxFile=512;
         of.lpstrFileTitle=NULL;
         of.nMaxFileTitle=0;
         of.lpstrInitialDir=NULL;
         of.lpstrTitle="Save As";
         of.Flags=OFN_EXPLORER|OFN_HIDEREADONLY|OFN_LONGNAMES
                 |OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST;
         of.nFileOffset=0;
         of.nFileExtension=0;
         of.lpstrDefExt="reg";
         of.lCustData=0;
         of.lpfnHook=NULL;
         of.lpTemplateName=NULL;
         if(GetSaveFileName(&of)){
            try{
               saveWff(hwnd,propFilters,filename);
            }catch(char*err){
               MessageBox(hwnd,err,"WebMask error",MB_ICONEXCLAMATION|MB_OK);
            }
         }
         break;
      }
      case ID_B_LOAD:
         char filename[512];
         *filename='\0';
         OPENFILENAME of;
         of.lStructSize=sizeof(OPENFILENAME);
         of.hwndOwner=hwnd;
         of.hInstance=NULL;
         of.lpstrFilter="WebMask filters (*.wff)\0*.wff\0All Files (*.*)\0*.*\0";
         of.lpstrCustomFilter=NULL;
         of.nMaxCustFilter=0;
         of.nFilterIndex=1;
         of.lpstrFile=filename;
         of.nMaxFile=512;
         of.lpstrFileTitle=NULL;
         of.nMaxFileTitle=0;
         of.lpstrInitialDir=NULL;
         of.lpstrTitle="Merge In";
         of.Flags=OFN_EXPLORER|OFN_HIDEREADONLY|OFN_LONGNAMES
                 |OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST;
         of.nFileOffset=0;
         of.nFileExtension=0;
         of.lpstrDefExt="reg";
         of.lCustData=0;
         of.lpfnHook=NULL;
         of.lpTemplateName=NULL;
         if(GetOpenFileName(&of)){
            try{
               if(!loadWff(hwnd,propFilters,filename,addToDlg,DWORD(this)))
                  MessageBox(hwnd,"All settings already in list","WebMask merge",
                         MB_ICONINFORMATION|MB_OK);
            }catch(char*err){
               MessageBox(hwnd,err,"WebMask error",MB_ICONEXCLAMATION|MB_OK);
            }
         }
         break;
      default:
         DialogPage::command(hwnd,id,hwndCtl,notify);
   }

   enable(hwnd);
   if(id==ID_B_NEW||id==ID_B_COPY){
      SetFocus(GetDlgItem(hwnd,ID_EB_FILTER_NAME));
////      int oldid=LOWORD(SendMessage(hwnd,DM_GETDEFID,0,0));
////      if(oldid!=IDOK){
//         SendMessage(hwnd,DM_SETDEFID,IDOK,0);
//         SendMessage(GetDlgItem(hwnd,ID_B_NEW),BM_SETSTYLE,BS_PUSHBUTTON,MAKELPARAM(TRUE,0));
//         SendMessage(GetDlgItem(hwnd,IDOK),BM_SETSTYLE,BS_DEFPUSHBUTTON,MAKELPARAM(TRUE,0));
////      }
   }
}

int FiltersDialogPage::apply(HWND hwnd,int,LPNMHDR){
   if(Button_GetCheck(GetDlgItem(hwnd,ID_CB_USEFILE)))
      strcpy(filterFile,oldFile);
   else *filterFile='\0';

   delete(activeFilters);
   activeFilters=new FilterList(*propFilters);

   save();
   return(PSNRET_NOERROR);
}

FiltersDialogPage filtersDlg;

