#define STRICT
#include <windows.h>
#include <windowsx.h>
#pragma hdrstop

#include <stdio.h>
#include <commctrl.h>

#include "WebMask.h"
#include "Dialog.h"
#include "Resource.h"

HWND generalWnd=NULL;

static DialogPage::HelpTopics helpIds[]={
   {ID_CB_ENABLED,      "auto.enabled"},
   {ID_T_PORT,          "auto.port"},
   {ID_EB_PORT,         "auto.port"},
   {ID_CB_START_ENABLED,"auto.startenabled"},
   {ID_CB_PROXY,        "auto.proxy"},
   {ID_T_PROXY_HOST,    "auto.proxy"},
   {ID_EB_PROXY_HOST,   "auto.proxy"},
   {ID_T_PROXY_PORT,    "auto.proxy"},
   {ID_EB_PROXY_PORT,   "auto.proxy"},
   {ID_CB_TUNNEL,       "auto.tunnel"},
   {ID_T_TUNNEL_HOST,   "auto.tunnel"},
   {ID_EB_TUNNEL_HOST,  "auto.tunnel"},
   {ID_T_TUNNEL_PORT,   "auto.tunnel"},
   {ID_EB_TUNNEL_PORT,  "auto.tunnel"}
};

class GeneralDialogPage:public DialogPage{
public:
   GeneralDialogPage():DialogPage(DIALOG_GENERAL,ICON_GENERAL,::helpIds,
      sizeof(::helpIds)/sizeof(*::helpIds),"general","general.html"){}
   virtual void save();
   virtual void load();
   virtual void enable(HWND);
   virtual BOOL init(HWND,HWND,LPARAM);
   virtual void exit(){generalWnd=NULL;};
   virtual void command(HWND hwnd,int id,HWND hwndCtl,UINT notify);
   virtual int apply(HWND hwnd,int,LPNMHDR pnmhdr);
};



void GeneralDialogPage::load(){
   HKEY hkey;
   if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                   KEY_READ,&hkey)!=ERROR_SUCCESS)
      hkey=NULL;
   LoadRegNumDef   (hkey,"enabled"    ,startEnabled ,TRUE                   );
   LoadRegNumDef   (hkey,"useProxy"   ,useprox      ,FALSE                  );
   LoadRegNumDef   (hkey,"useTunnel"  ,usetun       ,TRUE                   );
   LoadRegStringDef(hkey,"proxyHost"  ,proxname ,256,"www-cache.demon.co.uk");
   LoadRegNumDef   (hkey,"proxyPort"  ,proxport     ,(unsigned short)8080   );
   LoadRegStringDef(hkey,"tunnelHost" ,tunname  ,256,"127.0.0.1"            );
   LoadRegNumDef   (hkey,"tunnelPort" ,tunport      ,(unsigned short)9999   );
   LoadRegNumDef   (hkey,"port"       ,myPort       ,(unsigned short)8080   );

   if(hkey) RegCloseKey(hkey);
}

void GeneralDialogPage::save(){
   HKEY hkey;
   if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                     NULL,REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_WRITE,NULL,&hkey,NULL)==ERROR_SUCCESS){
      SaveRegNum   (hkey,"enabled"    ,startEnabled);
      SaveRegNum   (hkey,"useProxy"   ,useprox     );
      SaveRegNum   (hkey,"useTunnel"  ,usetun      );
      SaveRegString(hkey,"proxyHost"  ,proxname    );
      SaveRegNum   (hkey,"proxyPort"  ,proxport    );
      SaveRegString(hkey,"tunnelHost" ,tunname     );
      SaveRegNum   (hkey,"tunnelPort" ,tunport     );
      SaveRegNum   (hkey,"port"       ,myPort      );

      RegCloseKey(hkey);
   }
}

void GeneralDialogPage::enable(HWND hwnd){
   BOOL prox=Button_GetCheck(GetDlgItem(hwnd,ID_CB_PROXY));

   Static_Enable(GetDlgItem(hwnd,ID_T_PROXY_HOST ),prox);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_PROXY_HOST),prox);
   Static_Enable(GetDlgItem(hwnd,ID_T_PROXY_PORT ),prox);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_PROXY_PORT),prox);


   BOOL tun=Button_GetCheck(GetDlgItem(hwnd,ID_CB_TUNNEL));

   Static_Enable(GetDlgItem(hwnd,ID_T_TUNNEL_HOST ),tun);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_TUNNEL_HOST),tun);
   Static_Enable(GetDlgItem(hwnd,ID_T_TUNNEL_PORT ),tun);
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_TUNNEL_PORT),tun);
}

BOOL GeneralDialogPage::init(HWND hwnd,HWND,LPARAM){
   generalWnd=hwnd;
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ENABLED),         enabled);
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_START_ENABLED),   startEnabled);

   Button_SetCheck(GetDlgItem(hwnd,ID_CB_PROXY),     useprox);
   Edit_LimitText( GetDlgItem(hwnd,ID_EB_PROXY_HOST),255);
   Edit_SetText(   GetDlgItem(hwnd,ID_EB_PROXY_HOST),proxname);
   Edit_LimitText( GetDlgItem(hwnd,ID_EB_PROXY_PORT),5);
   char portStr[6];
   wsprintf(portStr,"%u",proxport);
   Edit_SetText(   GetDlgItem(hwnd,ID_EB_PROXY_PORT),portStr);

   Button_SetCheck(GetDlgItem(hwnd,ID_CB_TUNNEL),     usetun);
   Edit_LimitText( GetDlgItem(hwnd,ID_EB_TUNNEL_HOST),255);
   Edit_SetText(   GetDlgItem(hwnd,ID_EB_TUNNEL_HOST),tunname);
   Edit_LimitText( GetDlgItem(hwnd,ID_EB_TUNNEL_PORT),5);
   wsprintf(portStr,"%u",tunport);
   Edit_SetText(   GetDlgItem(hwnd,ID_EB_TUNNEL_PORT),portStr);

   Edit_LimitText( GetDlgItem(hwnd,ID_EB_PORT),5);
   wsprintf(portStr,"%u",myPort);
   Edit_SetText(   GetDlgItem(hwnd,ID_EB_PORT),portStr);

   enable(hwnd);
   return(TRUE);
}

void GeneralDialogPage::command(HWND hwnd,int id,HWND hwndCtl,UINT notify){
   switch(id){
      case ID_EB_PROXY_HOST:
      case ID_EB_PROXY_PORT:
      case ID_EB_TUNNEL_HOST:
      case ID_EB_TUNNEL_PORT:
      case ID_EB_PORT:
         if(notify==EN_CHANGE){
            PropSheet_Changed(hPropWnd,hwnd);
         }
         break;
      default:
         DialogPage::command(hwnd,id,hwndCtl,notify);
   }

   enable(hwnd);
}

int GeneralDialogPage::apply(HWND hwnd,int,LPNMHDR){
   enabled=Button_GetCheck(GetDlgItem(hwnd,ID_CB_ENABLED));
   startEnabled=Button_GetCheck(GetDlgItem(hwnd,ID_CB_START_ENABLED));

   useprox=Button_GetCheck(GetDlgItem(hwnd,ID_CB_PROXY));
   proxname[Edit_GetText(GetDlgItem(hwnd,ID_EB_PROXY_HOST),proxname,255)]='\0';
   char portStr[6];
   portStr[Edit_GetText(GetDlgItem(hwnd,ID_EB_PROXY_PORT),portStr,5)]='\0';
   sscanf(portStr,"%hu",&proxport);

   usetun=Button_GetCheck(GetDlgItem(hwnd,ID_CB_TUNNEL));
   tunname[Edit_GetText(GetDlgItem(hwnd,ID_EB_TUNNEL_HOST),tunname,255)]='\0';
   portStr[Edit_GetText(GetDlgItem(hwnd,ID_EB_TUNNEL_PORT),portStr,5)]='\0';
   sscanf(portStr,"%hu",&tunport);

   portStr[Edit_GetText(GetDlgItem(hwnd,ID_EB_PORT),portStr,5)]='\0';
   unsigned short oldPort=myPort;
   sscanf(portStr,"%hu",&myPort);
   if(oldPort!=myPort)
      startListening();

//   calcIP();

   ConnectCount::show();

   save();
   return(PSNRET_NOERROR);
}

GeneralDialogPage generalDlg;

