#define STRICT
#include <windows.h>
#include <windowsx.h>
#pragma hdrstop

#include <stdio.h>
#include <commctrl.h>

#include "WebMask.h"
#include "Dialog.h"
#include "Resource.h"

static DialogPage::HelpTopics helpIds[]={
   {ID_CB_ANON,            "auto.hidebrowser"},
   {ID_CB_ANON_USER_AGENT, "auto.hidebrowser"},
   {ID_CB_ANON_UA,         "auto.hidebrowser"},
   {ID_CB_ANON_REFERER,    "auto.hidebrowser"},
   {ID_CB_ANON_FROM,       "auto.hidebrowser"},
   {ID_RB_VIA_GENUINE,     "auto.viainformation"},
   {ID_RB_VIA_OVERRIDE,    "auto.viainformation"},
   {ID_RB_VIA_NONE,        "auto.viainformation"},
   {ID_EB_VIA,             "auto.viainformation"}
};

class PrivacyDialogPage:public DialogPage{
public:
   PrivacyDialogPage():DialogPage(DIALOG_ANON,ICON_ANON,::helpIds,
      sizeof(::helpIds)/sizeof(*::helpIds),"privacy","privacy.html"){}
   virtual void save();
   virtual void load();
   virtual void enable(HWND);
   virtual BOOL init(HWND,HWND,LPARAM);
//   virtual void exit();
   virtual void command(HWND hwnd,int id,HWND hwndCtl,UINT notify);
   virtual int apply(HWND hwnd,int,LPNMHDR pnmhdr);
};



void PrivacyDialogPage::load(){
   HKEY hkey;
   if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                   KEY_READ,&hkey)!=ERROR_SUCCESS)
      hkey=NULL;

   BOOL anon;
   LoadRegNumDef   (hkey,"anonymous"    ,anon         ,FALSE);
   LoadRegNumDef   (hkey,"anonUserAgent",anonUserAgent,anon );
   LoadRegNumDef   (hkey,"anonUA"       ,anonUA       ,anon );
   LoadRegNumDef   (hkey,"anonReferer"  ,anonReferer  ,anon );
   LoadRegNumDef   (hkey,"anonFrom"     ,anonFrom     ,anon );
//   LoadRegNumDef   (hkey,"anonymous"  ,anon         ,FALSE                  );

   LoadRegNumDef   (hkey,"viaType"    ,viaType      ,genuine                );
   char viaDefault[256];
   wsprintf(viaDefault,"1.0 %s:%u (WebMask/"WEBMASK_VERSION_NS")",myname,myPort);
   LoadRegStringDef(hkey,"viaText"    ,viaText  ,256,viaDefault);

   if(hkey) RegCloseKey(hkey);
}

void PrivacyDialogPage::save(){
   HKEY hkey;
   if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\PseudoFoo\\WebMask",0,
                     NULL,REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_WRITE,NULL,&hkey,NULL)==ERROR_SUCCESS){

      SaveRegNum   (hkey,"anonUserAgent",anonUserAgent);
      SaveRegNum   (hkey,"anonUA"       ,anonUA       );
      SaveRegNum   (hkey,"anonReferer"  ,anonReferer  );
      SaveRegNum   (hkey,"anonFrom"     ,anonFrom     );
      RegDeleteValue(hkey,"anonymous");
//      SaveRegNum   (hkey,"anonymous"  ,anon        );

      SaveRegNum   (hkey,"viaType"    ,viaType     );
      SaveRegString(hkey,"viaText"    ,viaText     );

      RegCloseKey(hkey);
   }
}

void PrivacyDialogPage::enable(HWND hwnd){
   BOOL via=Button_GetCheck(GetDlgItem(hwnd,ID_RB_VIA_OVERRIDE));
   Edit_Enable(  GetDlgItem(hwnd,ID_EB_VIA        ),via);
}

BOOL PrivacyDialogPage::init(HWND hwnd,HWND,LPARAM){
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON_USER_AGENT),anonUserAgent);
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON_UA        ),anonUA       );
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON_REFERER   ),anonReferer  );
   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON_FROM      ),anonFrom     );
   command(hwnd,ID_CB_ANON_USER_AGENT,NULL,0);
//   Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON),anon);

   Button_SetCheck(GetDlgItem(hwnd,ID_RB_VIA_GENUINE) ,viaType==genuine );
   Button_SetCheck(GetDlgItem(hwnd,ID_RB_VIA_OVERRIDE),viaType==override);
   Button_SetCheck(GetDlgItem(hwnd,ID_RB_VIA_NONE)    ,viaType==noVia   );

   Edit_LimitText(GetDlgItem(hwnd,ID_EB_VIA),255);
   Edit_SetText(  GetDlgItem(hwnd,ID_EB_VIA),viaText);

   enable(hwnd);
   return(TRUE);
}

void PrivacyDialogPage::command(HWND hwnd,int id,HWND hwndCtl,UINT notify){
   switch(id){
      case ID_CB_ANON:{
         bool ch=Button_GetCheck(hwndCtl)!=BST_CHECKED;
         Button_SetCheck(hwndCtl,ch);
         for(int i=ID_CB_ANON_USER_AGENT;i<=ID_CB_ANON_FROM;++i)
            Button_SetCheck(GetDlgItem(hwnd,i),ch);
         DialogPage::command(hwnd,id,hwndCtl,notify);
         break;
      }
      case ID_CB_ANON_USER_AGENT:
      case ID_CB_ANON_UA:
      case ID_CB_ANON_REFERER:
      case ID_CB_ANON_FROM:{
         bool c=false;
         bool u=false;
         for(int i=ID_CB_ANON_USER_AGENT;i<=ID_CB_ANON_FROM;++i){
            (Button_GetCheck(GetDlgItem(hwnd,i))?c:u)=true;
         }
         Button_SetCheck(GetDlgItem(hwnd,ID_CB_ANON),
            !c?BST_UNCHECKED:!u?BST_CHECKED:BST_INDETERMINATE);
         DialogPage::command(hwnd,id,hwndCtl,notify);
         break;
      }
      default:
         DialogPage::command(hwnd,id,hwndCtl,notify);
   }

   enable(hwnd);
}

int PrivacyDialogPage::apply(HWND hwnd,int,LPNMHDR){
//   anon=Button_GetCheck(GetDlgItem(hwnd,ID_CB_ANON));

   if(Button_GetCheck(GetDlgItem(hwnd,ID_RB_VIA_OVERRIDE)))
      viaType=override;
   else if(Button_GetCheck(GetDlgItem(hwnd,ID_RB_VIA_NONE)))
      viaType=noVia;
   else
      viaType=genuine;

   viaText[Edit_GetText(GetDlgItem(hwnd,ID_EB_VIA),viaText,255)]='\0';

   anonUserAgent=Button_GetCheck(GetDlgItem(hwnd,ID_CB_ANON_USER_AGENT));
   anonUA       =Button_GetCheck(GetDlgItem(hwnd,ID_CB_ANON_UA        ));
   anonReferer  =Button_GetCheck(GetDlgItem(hwnd,ID_CB_ANON_REFERER   ));
   anonFrom     =Button_GetCheck(GetDlgItem(hwnd,ID_CB_ANON_FROM      ));

   save();
   return(PSNRET_NOERROR);
}

PrivacyDialogPage privacyDlg;

