#include <iostream.h>
#include <fstream.h>

#include <winsock.h>

extern int sendCount;
extern int recvCount;

class WinSock{
   static WSADATA wsaData;
   static int count;
   bool ok;
public:
   WinSock():ok(false){ok=count++||WSAStartup(0x0101,&wsaData);}
   ~WinSock(){if(!--count) WSACleanup();}
   operator bool(){return(ok);}
};

class Socket:public WinSock{
   friend class SockSet;
public:
   SOCKET s;
private:   
   enum Type{DISCONNECTED,CONNECTED,LISTENING}type;
   sockaddr_in sa;
   bool closed_;
public:
   HANDLE winHandle(){return(HANDLE(s));}
   Socket():s(INVALID_SOCKET),type(DISCONNECTED),closed_(false){}
   ~Socket(){disconnect();}
   void disconnect(){
      if(s!=INVALID_SOCKET) closesocket(s);
      s=INVALID_SOCKET;
      type=DISCONNECTED;
      closed_=false;
   }
   int connect(const Socket&ls){
      disconnect();
      if(ls.s==INVALID_SOCKET||ls.type!=LISTENING) return(-1);
      int len=sizeof(sockaddr_in);
      s=accept(ls.s,(sockaddr*)&sa,&len);
      if(s==INVALID_SOCKET) return(WSAGetLastError());
      type=CONNECTED;
      return(0);
   }
   int connect(unsigned short port,unsigned long ip){
      disconnect();
      s=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
      if(s==INVALID_SOCKET) return(WSAGetLastError());
      sa.sin_family=AF_INET;
      sa.sin_port=htons(port);
      *(unsigned int*)&sa.sin_addr=ip;
      *(unsigned int*)sa.sin_zero=0;
      *(unsigned int*)(sa.sin_zero+4)=0;
      if(::connect(s,(sockaddr*)&sa,sizeof(sockaddr_in))==SOCKET_ERROR)
         return(WSAGetLastError());
      type=CONNECTED;
      return(0);
   }
   int listen(unsigned short port,unsigned long ip=INADDR_ANY){
      disconnect();
      s=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
      if(s==INVALID_SOCKET) return(WSAGetLastError());
      sa.sin_family=AF_INET;
      sa.sin_addr.s_addr=ip;
      sa.sin_port=htons(port);
      if(bind(s,(sockaddr*)&sa,sizeof(sockaddr_in))==SOCKET_ERROR)
         return(WSAGetLastError());
      ::listen(s,5);
      type=LISTENING;
      return(0);
   }

   bool closed(){return(type!=CONNECTED||closed_);}
   bool open(){return(!closed());}

   int setopt(int level,int optname,const char*optval,int optlen){
      return(setsockopt(s,level,optname,optval,optlen));
   }

   int recvRaw(char*buf,int maxlen){
      if(closed()) return(0);
      int n=::recv(s,buf,maxlen,0);
      if(!n) closed_=true;
      else if(n!=SOCKET_ERROR) sendCount+=n;
      return(n);
   }
   int sendRaw(char*buf,int maxlen){
      if(closed()) return(0);
      int n=::send(s,buf,maxlen,0);
      if(!n) closed_=true;
      else if(n!=SOCKET_ERROR) recvCount+=n;
      return(n);
   }
   int sendRaw(char*buf){return(sendRaw(buf,strlen(buf)));}

   int recv(char*buf,int maxlen);
   int send(char*buf,int maxlen);
   int send(char*buf){return(send(buf,strlen(buf)));}

   bool recvExact(char*buf,int len){return(recv(buf,len)==len);}
   bool sendExact(char*buf,int len){return(send(buf,len)==len);}
   bool sendExact(char*buf){return(sendExact(buf,strlen(buf)));}
};

class SockSet:public WinSock{
   friend class Select;
   fd_set fds;
public:
   SockSet(){zero();}
   SockSet(int){zero();}
   void zero(){FD_ZERO(&fds);}
   bool set(const Socket&s){return(FD_ISSET(s.s,&fds));}
   void add(const Socket&s);
   void del(const Socket&s);
};

class Select:public WinSock{
public:
   SockSet*r;
   SockSet*w;
   SockSet*e;
   Select(SockSet*r=NULL,SockSet*w=NULL,SockSet*e=NULL):r(r),w(w),e(e){}
   int wait(timeval*t,bool setres=false);
   int wait(bool setres=false){return(wait((timeval*)NULL,setres));}
   int wait(int sec,int usec=0,bool setres=false){
      return(wait(long(sec),long(usec),setres));
   }
   int wait(long sec,long usec=0L,bool setres=false){
      timeval t;
      t.tv_sec=sec;
      t.tv_usec=usec;
      return(wait(&t,setres));
   }
};

inline unsigned long getip(char*name){
   unsigned long ip=inet_addr(name);
   if(ip==INADDR_NONE){
      hostent*he=gethostbyname(name);
      if(he) ip=*(unsigned long*)he->h_addr;
   }
   return(ip);
}

inline unsigned long getip(char a,char b,char c,char d){
   return(a+(b<<8)+(c<<16)+(d<<24));
}
